import	os
import	unittest
from	time								import sleep
from	pygwarts.tests.hagrid				import MediumSet
from	pygwarts.irma.contrib				import LibraryContrib
from	pygwarts.irma.shelve				import LibraryShelf
from	pygwarts.hagrid.thrivables			import Tree
from	pygwarts.hagrid.thrivables			import Copse
from	pygwarts.hagrid.sprouts				import fssprout
from	pygwarts.hagrid.planting			import Flourish
from	pygwarts.hagrid.bloom.twigs			import Germination
from	pygwarts.hagrid.bloom.leafs			import Rejuvenation
from	pygwarts.hagrid.planting.leafs		import LeafGrowth
from	pygwarts.hagrid.planting.leafs		import LeafMove
from	pygwarts.hagrid.planting.twigs		import TwigThrive
from	pygwarts.hagrid.planting.peels		import GrowingPeel
from	pygwarts.hagrid.planting.peels		import ThrivingPeel
from	pygwarts.hagrid.planting.peeks		import BlindPeek
from	pygwarts.hagrid.planting.peeks		import DraftPeek
from	pygwarts.hagrid.cultivation.sifting	import SiftingController








class FirstCase(MediumSet):

	"""
		GrowingPeel or ThrivingPeel, DraftPeek or BlindPeek, some Germination -> Rejuvenation,
		regular or moving grows
		Triple sprout copse
		Triple bough copse
	"""


	@classmethod
	def tearDownClass(cls):
		cls.clean(cls)

		if	cls.clean_up:
			if	os.path.isfile(cls.MEDIUM_HANDLER_1): os.remove(cls.MEDIUM_HANDLER_1)

	@classmethod
	def setUpClass(cls):

		super().setUpClass()
		cls.make_loggy_file(cls, cls.MEDIUM_HANDLER_1)

	def setUp(self):
		class Forest(Copse):

			class loggy(LibraryContrib):

				handler		= self.MEDIUM_HANDLER_1
				init_level	= 10

			class TwoMain(Copse):

				class First(Tree):	bough = self.MEDIUM_SET_BOUGH_1		# This one must take all files
				class Second(Tree):	bough = self.MEDIUM_SET_BOUGH_3		# This one must have nothing left

				@GrowingPeel
				class thrive(TwigThrive):	pass
				class folders(Germination):	pass
				class leafs(SiftingController):
					include	= (

						( f"{self.MEDIUM_SET_SPROUT_1}/.+", f"{self.MEDIUM_SET_SPROUT_3}/.+" )
						if os.name == "posix" else
						(
							self.MEDIUM_SET_SPROUT_1.replace("\\","\\\\") + r"\\.+",
							self.MEDIUM_SET_SPROUT_3.replace("\\","\\\\") + r"\\.+"
						)
					)

			class Third(Tree):

				bough	= self.MEDIUM_SET_BOUGH_2
				@ThrivingPeel("weeds", to_peak=False)
				@BlindPeek("seeds", renew=False)
				class grow(LeafGrowth):		pass
				class seeds(LibraryShelf):

					grabbing	= self.MEDIUM_SET_SEEDS
					reclaiming	= self.MEDIUM_SET_SEEDS

			@GrowingPeel
			@DraftPeek(renew=False)
			class grow(LeafMove):			pass
			class files(Rejuvenation):		pass

			@fssprout(self.MEDIUM_SET_SPROUT_3)	# Order does matter cause every sprout pack it'self
			@fssprout(self.MEDIUM_SET_SPROUT_2)	# before previous sprout, so the nearest to dispatcher
			@fssprout(self.MEDIUM_SET_SPROUT_1)	# sprout will be processed first.
			class sync(Flourish):
				class twigs(SiftingController):
					exclude	= ( rf".+/table$", ) if os.name == "posix" else ( rf".+\\table$", )


		self.test_case = Forest


	def fthriving(self, dst :str) -> str :

		"""
			Makes destination file path according to hardcoded thriving
		"""

		return	os.path.join(
			self.MEDIUM_SET_BOUGH_2, "weeds", os.path.relpath(dst, self.MEDIUM_SET_BOUGH_2)
		)




	def test_first_flourish(self):

		self.test_case.loggy.init_name = "first_flourish"
		self.assertTrue(os.path.isdir(self.MEDIUM_SET_BOUGH_1))
		self.assertTrue(os.path.isdir(self.MEDIUM_SET_BOUGH_2))
		self.assertTrue(os.path.isdir(self.MEDIUM_SET_BOUGH_3))


		# First sprout
		self.assertTrue(os.path.isfile(self.f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))

		# Second sprout
		self.assertTrue(os.path.isfile(self.f_mss2_tasksschedule))
		self.assertTrue(os.path.isfile(self.f_mss2_explangraph))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardcleaning))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardwashing))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumcooking))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumhomework))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumphysicals))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyprocrastinating))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyreflections))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyphilosophizing))

		# Third sprout
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_milk))
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_cheese))
		self.assertTrue(os.path.isfile(self.f_mss3_k_f_meat))
		self.assertTrue(os.path.isfile(self.f_mss3_k_o_chicken))
		self.assertTrue(os.path.isfile(self.f_mss3_k_o_pie))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_bread))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb1))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb2))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb420))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb69))


		# First bough
		self.assertTrue(os.path.isfile(self.f_msb1_flower))
		# Second bough
		self.assertTrue(os.path.isdir(self.d_msb2_w))
		# Third bough
		self.assertTrue(os.path.isfile(self.f_msb3_almost))
		self.assertTrue(os.path.isdir(self.d_msb3_a))
		self.assertTrue(os.path.isdir(self.d_msb3_t))




		with self.assertLogs("first_flourish", 10) as case_loggy:

			self.test_case = self.test_case()
			self.test_case.sync()




		self.no_loggy_levels(case_loggy.output, 30,40,50)




		# First sprout
		self.assertFalse(os.path.isfile(self.f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))

		# Second sprout
		self.assertTrue(os.path.isfile(self.f_mss2_tasksschedule))
		self.assertTrue(os.path.isfile(self.f_mss2_explangraph))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardcleaning))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardwashing))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumcooking))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumhomework))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumphysicals))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyprocrastinating))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyreflections))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyphilosophizing))

		# Third sprout
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.f_mss3_k_o_pie))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_bread))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb1))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb2))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb420))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb69))


		# First bough
		self.assertTrue(os.path.isfile(self.f_msb1_flower))
		# Second bough
		self.assertTrue(os.path.isdir(self.d_msb2_w))
		# Third bough
		self.assertTrue(os.path.isfile(self.f_msb3_almost))
		self.assertTrue(os.path.isdir(self.d_msb3_a))
		self.assertTrue(os.path.isdir(self.d_msb3_t))




		# First sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_gc))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_goodbad))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_onion))

		# Second sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss2_c))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_explangraph))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_hardcleaning))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_hardwashing))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumcooking))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumhomework))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumphysicals))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyprocrastinating))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyreflections))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyphilosophizing))

		# Third sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss3_k))
		self.assertTrue(os.path.isdir(self.tg1_d_mss3_k_f))
		self.assertTrue(os.path.isdir(self.tg1_d_mss3_k_o))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_t))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_t_bc))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_f_milk))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_f_cheese))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_f_meat))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_o_chicken))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_o_pie))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_bread))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb1))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb2))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb420))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb69))




		# First sprout second bough
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_md)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_md_gc)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_good)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_notgood)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_bad)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_notbad)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_notnot)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_badbad)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_badgood)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_goodbad)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_rock)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_paper)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_scissors)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_gc_sadmovies)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_gc_dumbasses)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_gc_onion)))

		# Second sprout second bough
		self.assertTrue(os.path.isdir(self.fthriving(self.tg2_d_mss2_c)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_tasksschedule)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_explangraph)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_hardcleaning)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_hardwashing)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_mediumcooking)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_mediumhomework)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_mediumphysicals)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_easyprocrastinating)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_easyreflections)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_easyphilosophizing)))

		# Third sprout second bough
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k_f)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k_o)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k_t)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k_t_bc)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_f_milk)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_f_cheese)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_f_meat)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_o_chicken)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_o_pie)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_bread)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_crumb1)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_crumb2)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_crumb420)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_crumb69)))


		# Seeds for second bough
		self.assertEqual(len(self.test_case.Third.seeds()), 10)
		FirstCase.s_f_mss2_tasksschedule = self.test_case.Third.seeds()[
			self.f_mss2_tasksschedule
		]
		FirstCase.s_f_mss2_explangraph = self.test_case.Third.seeds()[
			self.f_mss2_explangraph
		]
		FirstCase.s_f_mss2_c_hardcleaning = self.test_case.Third.seeds()[
			self.f_mss2_c_hardcleaning
		]
		FirstCase.s_f_mss2_c_hardwashing = self.test_case.Third.seeds()[
			self.f_mss2_c_hardwashing
		]
		FirstCase.s_f_mss2_c_mediumcooking = self.test_case.Third.seeds()[
			self.f_mss2_c_mediumcooking
		]
		FirstCase.s_f_mss2_c_mediumhomework = self.test_case.Third.seeds()[
			self.f_mss2_c_mediumhomework
		]
		FirstCase.s_f_mss2_c_mediumphysicals = self.test_case.Third.seeds()[
			self.f_mss2_c_mediumphysicals
		]
		FirstCase.s_f_mss2_c_easyprocrastinating = self.test_case.Third.seeds()[
			self.f_mss2_c_easyprocrastinating
		]
		FirstCase.s_f_mss2_c_easyreflections = self.test_case.Third.seeds()[
			self.f_mss2_c_easyreflections
		]
		FirstCase.s_f_mss2_c_easyphilosophizing = self.test_case.Third.seeds()[
			self.f_mss2_c_easyphilosophizing
		]
		self.assertIsInstance(FirstCase.s_f_mss2_tasksschedule, int)
		self.assertIsInstance(FirstCase.s_f_mss2_explangraph, int)
		self.assertIsInstance(FirstCase.s_f_mss2_c_hardcleaning, int)
		self.assertIsInstance(FirstCase.s_f_mss2_c_hardwashing, int)
		self.assertIsInstance(FirstCase.s_f_mss2_c_mediumcooking, int)
		self.assertIsInstance(FirstCase.s_f_mss2_c_mediumhomework, int)
		self.assertIsInstance(FirstCase.s_f_mss2_c_mediumphysicals, int)
		self.assertIsInstance(FirstCase.s_f_mss2_c_easyprocrastinating, int)
		self.assertIsInstance(FirstCase.s_f_mss2_c_easyreflections, int)
		self.assertIsInstance(FirstCase.s_f_mss2_c_easyphilosophizing, int)





		# First sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md_gc))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_goodbad))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_onion))

		# Second sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss2_c))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_explangraph))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_hardcleaning))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_hardwashing))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumcooking))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumhomework))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumphysicals))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyprocrastinating))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyreflections))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyphilosophizing))

		# Third sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss3_k))
		self.assertTrue(os.path.isdir(self.tg3_d_mss3_k_f))
		self.assertTrue(os.path.isdir(self.tg3_d_mss3_k_o))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_t))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_t_bc))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_o_pie))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_bread))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb1))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb2))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb420))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb69))
















	def test_no_touch_flourish(self):

		sleep(1.1)
		self.test_case.loggy.init_name = "no_touch_flourish"


		# First sprout
		self.assertFalse(os.path.isfile(self.f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))

		# Second sprout
		self.assertTrue(os.path.isfile(self.f_mss2_tasksschedule))
		self.assertTrue(os.path.isfile(self.f_mss2_explangraph))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardcleaning))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardwashing))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumcooking))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumhomework))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumphysicals))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyprocrastinating))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyreflections))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyphilosophizing))

		# Third sprout
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.f_mss3_k_o_pie))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_bread))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb1))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb2))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb420))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb69))


		# First bough
		self.assertTrue(os.path.isfile(self.f_msb1_flower))
		# Second bough
		self.assertTrue(os.path.isdir(self.d_msb2_w))
		# Third bough
		self.assertTrue(os.path.isfile(self.f_msb3_almost))
		self.assertTrue(os.path.isdir(self.d_msb3_a))
		self.assertTrue(os.path.isdir(self.d_msb3_t))




		with self.assertLogs("no_touch_flourish", 10) as case_loggy:

			self.test_case = self.test_case()
			self.test_case.sync()




		self.no_loggy_levels(case_loggy.output, 30,40,50)




		# First sprout
		self.assertFalse(os.path.isfile(self.f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))

		# Second sprout
		self.assertTrue(os.path.isfile(self.f_mss2_tasksschedule))
		self.assertTrue(os.path.isfile(self.f_mss2_explangraph))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardcleaning))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardwashing))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumcooking))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumhomework))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumphysicals))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyprocrastinating))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyreflections))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyphilosophizing))

		# Third sprout
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.f_mss3_k_o_pie))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_bread))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb1))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb2))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb420))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb69))


		# First bough
		self.assertTrue(os.path.isfile(self.f_msb1_flower))
		# Second bough
		self.assertTrue(os.path.isdir(self.d_msb2_w))
		# Third bough
		self.assertTrue(os.path.isfile(self.f_msb3_almost))
		self.assertTrue(os.path.isdir(self.d_msb3_a))
		self.assertTrue(os.path.isdir(self.d_msb3_t))




		# First sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_gc))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_goodbad))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_onion))

		# Second sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss2_c))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_explangraph))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_hardcleaning))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_hardwashing))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumcooking))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumhomework))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumphysicals))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyprocrastinating))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyreflections))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyphilosophizing))

		# Third sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss3_k))
		self.assertTrue(os.path.isdir(self.tg1_d_mss3_k_f))
		self.assertTrue(os.path.isdir(self.tg1_d_mss3_k_o))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_t))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_t_bc))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_f_milk))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_f_cheese))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_f_meat))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_o_chicken))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_o_pie))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_bread))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb1))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb2))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb420))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb69))




		# First sprout second bough
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_md)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_md_gc)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_good)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_notgood)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_bad)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_notbad)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_notnot)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_badbad)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_badgood)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_goodbad)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_rock)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_paper)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_scissors)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_gc_sadmovies)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_gc_dumbasses)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_gc_onion)))

		# Second sprout second bough
		self.assertTrue(os.path.isdir(self.fthriving(self.tg2_d_mss2_c)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_tasksschedule)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_explangraph)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_hardcleaning)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_hardwashing)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_mediumcooking)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_mediumhomework)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_mediumphysicals)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_easyprocrastinating)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_easyreflections)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_easyphilosophizing)))

		# Third sprout second bough
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k_f)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k_o)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k_t)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k_t_bc)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_f_milk)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_f_cheese)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_f_meat)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_o_chicken)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_o_pie)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_bread)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_crumb1)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_crumb2)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_crumb420)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_crumb69)))


		# Seeds for second bough
		self.assertEqual(len(self.test_case.Third.seeds()), 10)
		self.assertEqual(

			self.test_case.Third.seeds()[self.f_mss2_tasksschedule],
			FirstCase.s_f_mss2_tasksschedule
		)
		self.assertEqual(

			self.test_case.Third.seeds()[self.f_mss2_explangraph],
			FirstCase.s_f_mss2_explangraph
		)
		self.assertEqual(

			self.test_case.Third.seeds()[self.f_mss2_c_hardcleaning],
			FirstCase.s_f_mss2_c_hardcleaning
		)
		self.assertEqual(

			self.test_case.Third.seeds()[self.f_mss2_c_hardwashing],
			FirstCase.s_f_mss2_c_hardwashing
		)
		self.assertEqual(

			self.test_case.Third.seeds()[self.f_mss2_c_mediumcooking],
			FirstCase.s_f_mss2_c_mediumcooking
		)
		self.assertEqual(

			self.test_case.Third.seeds()[self.f_mss2_c_mediumhomework],
			FirstCase.s_f_mss2_c_mediumhomework
		)
		self.assertEqual(

			self.test_case.Third.seeds()[self.f_mss2_c_mediumphysicals],
			FirstCase.s_f_mss2_c_mediumphysicals
		)
		self.assertEqual(

			self.test_case.Third.seeds()[self.f_mss2_c_easyprocrastinating],
			FirstCase.s_f_mss2_c_easyprocrastinating
		)
		self.assertEqual(

			self.test_case.Third.seeds()[self.f_mss2_c_easyreflections],
			FirstCase.s_f_mss2_c_easyreflections
		)
		self.assertEqual(

			self.test_case.Third.seeds()[self.f_mss2_c_easyphilosophizing],
			FirstCase.s_f_mss2_c_easyphilosophizing
		)




		# First sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md_gc))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_goodbad))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_onion))

		# Second sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss2_c))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_explangraph))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_hardcleaning))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_hardwashing))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumcooking))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumhomework))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumphysicals))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyprocrastinating))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyreflections))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyphilosophizing))

		# Third sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss3_k))
		self.assertTrue(os.path.isdir(self.tg3_d_mss3_k_f))
		self.assertTrue(os.path.isdir(self.tg3_d_mss3_k_o))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_t))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_t_bc))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_o_pie))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_bread))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb1))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb2))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb420))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb69))
















	def test_touch_flourish(self):

		sleep(1.1)
		self.test_case.loggy.init_name = "touch_flourish"
		self.fmake(self.f_mss2_tasksschedule, "1. get up\n2. get sad")
		self.fmake(self.f_mss2_explangraph, "FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF")
		self.fmake(self.f_mss2_c_hardcleaning, "especially vacuum")
		self.fmake(self.f_mss2_c_hardwashing, "dishes so dishes")
		self.fmake(self.f_mss2_c_mediumcooking, "who do you think i am, a chemist?")
		self.fmake(self.f_mss2_c_mediumhomework, "my son homework, ofcourse")
		self.fmake(self.f_mss2_c_mediumphysicals, "what the flip is that?")
		self.fmake(self.f_mss2_c_easyprocrastinating, "the easiest thing ever")
		self.fmake(self.f_mss2_c_easyreflections, "litlle harder but still easy")
		self.fmake(self.f_mss2_c_easyphilosophizing, "not easy at all, but why not")


		# First sprout
		self.assertFalse(os.path.isfile(self.f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))

		# Second sprout
		self.assertTrue(os.path.isfile(self.f_mss2_tasksschedule))
		self.assertTrue(os.path.isfile(self.f_mss2_explangraph))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardcleaning))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardwashing))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumcooking))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumhomework))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumphysicals))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyprocrastinating))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyreflections))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyphilosophizing))

		# Third sprout
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.f_mss3_k_o_pie))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_bread))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb1))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb2))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb420))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb69))


		# First bough
		self.assertTrue(os.path.isfile(self.f_msb1_flower))
		# Second bough
		self.assertTrue(os.path.isdir(self.d_msb2_w))
		# Third bough
		self.assertTrue(os.path.isfile(self.f_msb3_almost))
		self.assertTrue(os.path.isdir(self.d_msb3_a))
		self.assertTrue(os.path.isdir(self.d_msb3_t))




		with self.assertLogs("touch_flourish", 10) as case_loggy:

			self.test_case = self.test_case()
			self.test_case.sync()




		self.no_loggy_levels(case_loggy.output, 30,40,50)




		# First sprout
		self.assertFalse(os.path.isfile(self.f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.f_mss1_md_gc_onion))
		self.assertTrue(os.path.isdir(self.d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.d_mss1_c_goodbad))

		# Second sprout
		self.assertTrue(os.path.isfile(self.f_mss2_tasksschedule))
		self.assertTrue(os.path.isfile(self.f_mss2_explangraph))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardcleaning))
		self.assertTrue(os.path.isfile(self.f_mss2_c_hardwashing))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumcooking))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumhomework))
		self.assertTrue(os.path.isfile(self.f_mss2_c_mediumphysicals))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyprocrastinating))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyreflections))
		self.assertTrue(os.path.isfile(self.f_mss2_c_easyphilosophizing))

		# Third sprout
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.f_mss3_k_o_pie))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_bread))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb1))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb2))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb420))
		self.assertTrue(os.path.isfile(self.f_mss3_k_t_bc_crumb69))


		# First bough
		self.assertTrue(os.path.isfile(self.f_msb1_flower))
		# Second bough
		self.assertTrue(os.path.isdir(self.d_msb2_w))
		# Third bough
		self.assertTrue(os.path.isfile(self.f_msb3_almost))
		self.assertTrue(os.path.isdir(self.d_msb3_a))
		self.assertTrue(os.path.isdir(self.d_msb3_t))




		# First sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_md_gc))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg1_d_mss1_c_goodbad))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_rock))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_paper))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_scissors))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_sadmovies))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_dumbasses))
		self.assertTrue(os.path.isfile(self.tg1_f_mss1_md_gc_onion))

		# Second sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss2_c))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_explangraph))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_hardcleaning))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_hardwashing))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumcooking))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumhomework))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_mediumphysicals))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyprocrastinating))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyreflections))
		self.assertFalse(os.path.isfile(self.tg1_f_mss2_c_easyphilosophizing))

		# Third sprout first bough
		self.assertTrue(os.path.isdir(self.tg1_d_mss3_k))
		self.assertTrue(os.path.isdir(self.tg1_d_mss3_k_f))
		self.assertTrue(os.path.isdir(self.tg1_d_mss3_k_o))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_t))
		self.assertFalse(os.path.isdir(self.tg1_d_mss3_k_t_bc))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_f_milk))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_f_cheese))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_f_meat))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_o_chicken))
		self.assertTrue(os.path.isfile(self.tg1_f_mss3_k_o_pie))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_bread))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb1))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb2))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb420))
		self.assertFalse(os.path.isfile(self.tg1_f_mss3_k_t_bc_crumb69))




		# First sprout second bough
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_md)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_md_gc)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_good)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_notgood)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_bad)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_notbad)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_notnot)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_badbad)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_badgood)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss1_c_goodbad)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_rock)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_paper)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_scissors)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_gc_sadmovies)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_gc_dumbasses)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss1_md_gc_onion)))

		# Second sprout second bough
		self.assertTrue(os.path.isdir(self.fthriving(self.tg2_d_mss2_c)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_tasksschedule)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_explangraph)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_hardcleaning)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_hardwashing)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_mediumcooking)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_mediumhomework)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_mediumphysicals)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_easyprocrastinating)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_easyreflections)))
		self.assertTrue(os.path.isfile(self.fthriving(self.tg2_f_mss2_c_easyphilosophizing)))

		# Third sprout second bough
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k_f)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k_o)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k_t)))
		self.assertFalse(os.path.isdir(self.fthriving(self.tg2_d_mss3_k_t_bc)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_f_milk)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_f_cheese)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_f_meat)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_o_chicken)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_o_pie)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_bread)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_crumb1)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_crumb2)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_crumb420)))
		self.assertFalse(os.path.isfile(self.fthriving(self.tg2_f_mss3_k_t_bc_crumb69)))


		# Seeds for second bough
		self.assertEqual(len(self.test_case.Third.seeds()), 10)
		self.assertGreater(

			self.test_case.Third.seeds()[self.f_mss2_tasksschedule],
			FirstCase.s_f_mss2_tasksschedule
		)
		self.assertGreater(

			self.test_case.Third.seeds()[self.f_mss2_explangraph],
			FirstCase.s_f_mss2_explangraph
		)
		self.assertGreater(

			self.test_case.Third.seeds()[self.f_mss2_c_hardcleaning],
			FirstCase.s_f_mss2_c_hardcleaning
		)
		self.assertGreater(

			self.test_case.Third.seeds()[self.f_mss2_c_hardwashing],
			FirstCase.s_f_mss2_c_hardwashing
		)
		self.assertGreater(

			self.test_case.Third.seeds()[self.f_mss2_c_mediumcooking],
			FirstCase.s_f_mss2_c_mediumcooking
		)
		self.assertGreater(

			self.test_case.Third.seeds()[self.f_mss2_c_mediumhomework],
			FirstCase.s_f_mss2_c_mediumhomework
		)
		self.assertGreater(

			self.test_case.Third.seeds()[self.f_mss2_c_mediumphysicals],
			FirstCase.s_f_mss2_c_mediumphysicals
		)
		self.assertGreater(

			self.test_case.Third.seeds()[self.f_mss2_c_easyprocrastinating],
			FirstCase.s_f_mss2_c_easyprocrastinating
		)
		self.assertGreater(

			self.test_case.Third.seeds()[self.f_mss2_c_easyreflections],
			FirstCase.s_f_mss2_c_easyreflections
		)
		self.assertGreater(

			self.test_case.Third.seeds()[self.f_mss2_c_easyphilosophizing],
			FirstCase.s_f_mss2_c_easyphilosophizing
		)




		# First sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_md_gc))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_good))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_bad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_notnot))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badbad))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_badgood))
		self.assertTrue(os.path.isdir(self.tg3_d_mss1_c_goodbad))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_rock))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_paper))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_scissors))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_sadmovies))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_dumbasses))
		self.assertFalse(os.path.isfile(self.tg3_f_mss1_md_gc_onion))

		# Second sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss2_c))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_tasksschedule))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_explangraph))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_hardcleaning))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_hardwashing))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumcooking))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumhomework))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_mediumphysicals))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyprocrastinating))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyreflections))
		self.assertFalse(os.path.isfile(self.tg3_f_mss2_c_easyphilosophizing))

		# Third sprout third bough
		self.assertTrue(os.path.isdir(self.tg3_d_mss3_k))
		self.assertTrue(os.path.isdir(self.tg3_d_mss3_k_f))
		self.assertTrue(os.path.isdir(self.tg3_d_mss3_k_o))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_t))
		self.assertFalse(os.path.isdir(self.tg3_d_mss3_k_t_bc))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_milk))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_cheese))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_f_meat))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_o_chicken))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_o_pie))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_bread))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb1))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb2))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb420))
		self.assertFalse(os.path.isfile(self.tg3_f_mss3_k_t_bc_crumb69))








if	__name__ == "__main__" : unittest.main(verbosity=2)










import	re
from	typing					import List
from	pygwarts.magical.chests	import KeyChest








class LibraryAccess(KeyChest):

	"""
		irma.access main object, that represents the container for any accessing objects. Original design
		implies next library access algorithm:
			1.	LibraryAccess object is the top level for access;
			2.	LibraryVolume objects are declared under the top LibraryAccess and represents the volumes,
				or file locations to be accessed;
			3.	VolumeBookmark objects are declared either under the top LibraryAccess or under
				corresponding LibraryVolume, to organize handling of volumes access (it will be common
				bookmarks that are declared under the top LibraryAccess, means will be available for every
				LibraryVolume);
			4.	AccessHandler objects are declared under the corresponding VolumeBookmark objects and are
				defined the way they should handle information they will be supplied by the triggered
				bookmarks they declared under (or common);
			5.	AccessInducer objects are declared under corresponding AccessHandler and are defined the
				way the data handled by AccessHandler they declared under might be induced (or viewed) - it
				must return a string that will represent the view;
			6.	VolumeAnnex objects are declared under corresponding LibraryVolume objects as "Annex"
				member and organizes the volume data induce (or view) by collecting all volume's
				AccessInducer's strings into one concatenated string to be returned;
			7.	LibraryAnnex object declared under the top LibraryAccess and organizes first every volume
				access by it's API, and then tries to invoke every accessed volume's "Annex" to produce
				the final concatenation of all volumes "Annex" concatenations to return it.
			8.	Any other objects might be declared under the top LibraryAccess to be integrated into
				mutable chain and to be reachable for every access member.
	"""

	def __init__(self, *args, **kwargs):

		super().__init__(*args, **kwargs)
		self.loggy.debug(f"Library access point created")


	def compile_triggers(self, triggers :List[str]) -> re.Pattern | None :

		"""
			Utility method for LibraryVolume objects to compile their VolumeBookmark triggers into one big
			regex. Accepts list "triggers" with every item is a string that represents a "trigger" field
			for a VolumeBookmark object. The final regex will be formed from every non empty string from
			"triggers" as an individual group. Returns compiled regex if at least one group formed. Returns
			None if "triggers" not a list or no groups were formed.
		"""

		if	isinstance(triggers, list) and (pattern := "("):

			for trigger in triggers:
				if	isinstance(trigger, str) and len(trigger):


					pattern += f"{trigger})|("
					self.loggy.debug(f"Compiling trigger \"{trigger}\"")
				else:
					self.loggy.debug(f"Skipping invalid trigger \"{trigger}\"")


			if		1 <len(pattern): return re.compile(pattern.rstrip("|("))
			else:	self.loggy.debug("No triggers compiled")
		else:		self.loggy.debug(f"Invalid triggers {type(triggers)}")










from re										import Pattern
from typing									import List
from pygwarts.magical.philosophers_stone	import Transmutable
from pygwarts.magical.spells				import flagrate
from pygwarts.irma.access					import LibraryAccess
from pygwarts.irma.access.volume			import LibraryVolume
from pygwarts.irma.access.bookmarks			import VolumeBookmark








class LibraryAnnex(Transmutable):

	"""
		pygwarts.irma helper class that represents the dispatcher of entire LibraryAccess object it
		declared under. When all LibraryVolume objects, that are mapped as "volume" in upper LibraryAccess
		KeyChest, will be accessed, the further process might be divided by two phases:
			1. Actual processing of every mapped LibraryVolume object, by full staging the "volume":
				- getting "volume" range as a verification;
				- generating lines of opened "volume" file;
				- generating responses of lines parsing with triggers compiled from bookmarks;
				- updating every corresponding bookmark by line that triggered.
		This phase also implies creating a runtime mapping of successfully processed "volumes" with
		lists of corresponding bookmarks, which will be created the way LibraryAccess's bookmarks
		(or common) will preceded LibraryVolume's bookmarks (or local).
			2. After all volumes were processed and gathered, their own "Annex" will be seeked and called,
		in order to fetch volume's annex string. After all such string will be obtained and concatenated,
		the final string will be returned.
		Returns None if either LibraryAnnex object is not a part of library or if final annex string
		length is zero.
	"""

	def __call__(self) -> str | None :


		if	isinstance(liblink := getattr(self, "_UPPER_LAYER", None), LibraryAccess):
			self.loggy.debug(f"Fetching library {liblink} annex")


			processed_volumes = {}
			common = liblink.keysof("bookmark")
			self.loggy.debug("Processing volumes")


			for volume in liblink.keysof("volume"):
				if	isinstance(inrange := volume.get_range(), Pattern):


					# Every "volume" will have it's own list of bookmarks, ordered the way common bookmarks
					# will preceed locals for proper "volume" viewing.
					currents = common + volume.keysof("bookmark")
					triggers = liblink.compile_triggers([ b.trigger for b in currents ])


					# Every "volume" will also have common bookmarks injected to be all included to be
					# handled in "volume" processing.
					for bookmark in common:

						volume(bookmark, "bookmark")
						volume(bookmark.trigger, bookmark)


					for line in volume.g_reading():
						if	inrange.match(line):

							self.loggy.debug(f"Triggering line \"{line}\"")

							for response in volume.g_triggering(line, triggers):
								if	(bookmark := volume[response]) is not None:
									bookmark.update(line, volume)


					processed_volumes[volume] = currents
					self.loggy.debug(f"Volume {volume} processed")


			self.loggy.debug("Viewing volumes")
			lib_resume = str()


			# As of Python3.7+ dictionaries are ordered, "processed_volumes" will guarantee "Annex"
			# invocation, and hence final annex string order will match the order they were declared.
			# Original design implies every volume must implement "Annex" callable, which will be
			# supplied with list of bookmarks to view and must return current volume annex string.
			for volume,bookmarks in processed_volumes.items():
				if	callable(getattr(volume, "Annex", None)):

					annex = volume.Annex(bookmarks)
					if	isinstance(annex, str): lib_resume += annex
				else:	self.loggy.debug(f"Volume {volume} annex cannot be fetched")


			self.loggy.debug(f"Library {liblink} annex {(RL := len(lib_resume))} symbol{flagrate(RL)}")


			if	RL: return lib_resume
		else:	self.loggy.debug("Library not found")








class VolumeAnnex(Transmutable):

	"""
		pygwarts.irma helper class that represents the dispatcher of a LibraryVolume object bookmarks
		viewing. Accepts a list of VolumeBookmark objects "bookmarks", that must be relevant to current
		LibraryVolume object and will be viewed in determined order, instead of order they were putted.
		For every relevant bookmark "view" method will be called and it's string results will be
		concatenated as a final string to be returned. Every other than VolumeBookmark "bookmarks"
		object, or every "view" result other than string will be skipped and reported by corresponding
		loggy message. Returns None in any other case, even if result string length is zero.
		Original design implies this object will be declared under every LibraryVolume object as "Annex".
	"""

	def __call__(self, bookmarks :List[VolumeBookmark]) -> str | None :

		if	isinstance(volume := getattr(self, "_UPPER_LAYER", None), LibraryVolume):
			self.loggy.debug(f"Fetching volume {volume} annex")


			if	isinstance(bookmarks, list) and not (volume_view := str()):
				for bookmark in bookmarks:


					if	isinstance(bookmark, VolumeBookmark):
						if	callable(view := getattr(bookmark, "view", None)):


							match (current := view(volume)):
								case str() if (LC := len(current)):

									volume_view += current
									self.loggy.debug(f"{bookmark} bookmark view {LC} symbol{flagrate(LC)}")

								case None:	self.loggy.debug(f"{bookmark} view is None")
								case _:		self.loggy.debug(f"{bookmark} view is not a string")


						else:	self.loggy.debug(f"{bookmark} is not viewable")
					else:		self.loggy.debug(f"Invalid bookmark {bookmark}")


				self.loggy.debug(f"Volume {volume} annex {(LVV := len(volume_view))} symbol{flagrate(LVV)}")


				if	LVV : return volume_view
			else:	self.loggy.debug("Provided bookmarks invalid")
		else:		self.loggy.debug("Library not found")










import re








VALID_IP4	= r"((25[0-5]|2[0-4]\d|1\d\d|0?\d\d|00\d|\d)\.){3}(25[0-5]|2[0-4]\d|1\d\d|0?\d\d|00\d|\d)"
P_VALID_IP4	= re.compile(VALID_IP4)
G_VALID_IP4	= str(

	r"(?P<octet1>25[0-5]|2[0-4]\d|1\d\d|0?\d\d|00\d|\d)\."
	r"(?P<octet2>25[0-5]|2[0-4]\d|1\d\d|0?\d\d|00\d|\d)\."
	r"(?P<octet3>25[0-5]|2[0-4]\d|1\d\d|0?\d\d|00\d|\d)\."
	r"(?P<octet4>25[0-5]|2[0-4]\d|1\d\d|0?\d\d|00\d|\d)"
)
GP_VALID_IP4 = re.compile(G_VALID_IP4)




def is_valid_ip4(addr :str) -> bool | None :

	"""
		Helper function that verifies "addr" string argument is a valid ip version 4 address.
		Verification goes the classic way by splitting to parts and validating it is 0-255 integer
		range octets, without 0-padding. Returns True if "addr" consists of exactly four valid octets.
		Returns False if at least one part is not a valid or is 0-padded octet. Returns None for the rest.
	"""

	if	isinstance(addr, str) and len(S := addr.split(".")) == 4:

		for octet in S:

			try:	octet_int = int(octet)
			except:	return
			else:

				if	octet_int <0 or 255 <octet_int or len(str(octet_int)) != len(octet): return False
		return	True




def validate_ip4(addr :str) -> str | None :

	"""
		Helper function that takes "addr" string and returns it's sanitized version (no padding zeroes)
		if it represents valid ip version 4 address. Utilizes "is_valid_ip4" function for "addr"
		validation. Returns None if "is_valid_ip4" doesn't return True.
	"""

	if	isinstance(addr, str) and len(S := addr.split(".")) == 4:

		try:

			if	is_valid_ip4(sanit := ".".join( str(octet) for octet in map(int,S) if -1 <octet <256 )):
				return sanit
		except:	return










from typing									import Literal
from typing									import Callable
from pygwarts.magical.philosophers_stone	import Transmutable
from pygwarts.magical.chests				import KeyChest
from pygwarts.magical.spells				import patronus
from pygwarts.irma.access					import LibraryAccess
from pygwarts.irma.access.volume			import LibraryVolume








class VolumeBookmark(KeyChest):

	"""
		irma.access core super class that serves as a dispatcher for handlers and inducers. As the library
		access implies, for every LibraryVolume object to be "accessed", or in other words to be processed,
		corresponding AccessHandler objects will do the thing. After LibraryVolume is done with processing
		it's content, corresponding AccessInducer will represent processed data. The VolumeBookmark object
		is the way AccessHandler and AccessInducer objects will communicate with VolumeBookmark. By
		declaring right under corresponding LibraryVolume object, VolumeBookmark will be able to put itself
		to it in two ways:
			1. as VolumeBookmark: "bookmark" mapping to ensure it might be accessed from LibraryVolume;
			2. as VolumeBookmark.trigger: VolumeBookmark mapping to allow access by it's trigger in text.
		It is also possible to declare VolumeBookmark under the toppest LibraryAccess object. In this case
		it will put itself as VolumeBookmark: "bookmark" mapping and will be reachable to every
		LibraryVolume in current library, so it will be named a common bookmark. When the access to library
		will occur, every LibraryVolume object will read the files it is points to and exposes every line
		of text it encounters. By the field "trigger", that must contain a string, every assigned volume's
		or common VolumeBookmark might be triggered, if it's "trigger" string is a substring of line being
		processing. After such triggering, method "update" will be invoked for every triggered
		VolumeBookmark, which means AccessHandler objects will be invoked. At the end of the access, method
		"view" might be invoked to, in turn, invoke every AccessInducer object.
	"""

	trigger	:str

	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)


		if	isinstance(bookmark_trigger := getattr(self, "trigger", None), str):
			if	self.library_assignment(bookmark_trigger, getattr(self, "_UPPER_LAYER", None)): return
		else:	self.loggy.debug(f"Incomplete bookmark")


		self.loggy.info(f"Bookmark {self} not assigned to library")




	def library_assignment(	self,
							trigger	:str,
							liblink	:LibraryAccess | LibraryVolume | None
						)-> Literal[True] | None :

		"""
			Helper method that checks "liblink" object to be either LibraryVolume or LibraryAccess type.
			For a LibraryVolume it will put itself in two ways:
				1. as VolumeBookmark: "bookmark" mapping to ensure it might be accessed from LibraryVolume;
				2. as VolumeBookmark.trigger: VolumeBookmark mapping to allow access by it's trigger in text.
			For a LibraryAccess it will put itself as VolumeBookmark: "bookmark" mapping so it will be
			reachable to every LibraryVolume in current library. Returns True in case of successful
			assignment, or None otherwise.
		"""

		match liblink:
			case LibraryAccess():

				liblink(self, "bookmark")
				self.loggy.debug(f"Assigned to library {liblink}")
				return	True


			case LibraryVolume():

				liblink(self, "bookmark")
				liblink(trigger, self)
				self.loggy.debug(f"Assigned to volume {liblink}")
				return	True


			case None	: self.loggy.debug("Upper layer not found")
			case _		: self.loggy.debug("Invalid upper layer")




	def update(self, line :str, volume :LibraryVolume):

		"""
			Helper method, that accepts current "line" being processing and current LibraryVolume "volume"
			object being processing, and invokes AccessHandler objects, that was registered to current
			VolumeBookmark object, passing both arguments. As every AccessHandler invocation is putted
			in try block, any Exception caught will be logged and skipped. Doesn't suppose to return
			anything.
		"""

		for handler in self.keysof("handler"):

			try:	handler(line, volume)
			except	Exception as E : self.loggy.debug(f"{handler} failed due to {patronus(E)}")




	def view(self, volume :LibraryVolume) -> str | None :

		"""
			Helper method, that accepts current LibraryVolume "volume" object being processing, and invokes
			AccessInducer objects, that was registered to current VolumeBookmark object, passing "volume".
			Assumes that every AccessInducer will result a string. Will collect and concatenate all
			AccessInducer result strings and return it. Returns None if no string fetched from AccessInducer
			objects. As every AccessInducer invocation is putted in try block, any Exception caught will be
			logged and skipped.
		"""

		current_view = str()

		for inducer in self.keysof("inducer"):

			# Inducers must take volume and use it to index in it to maintain it's recaps/counters
			try:	current_view += inducer(volume)
			except	Exception as E : self.loggy.debug(f"{inducer} failed due to {patronus(E)}")


		if		current_view: return current_view
		else:	self.loggy.debug(f"No view volume {volume}")










from typing									import Any
from typing									import Callable
from pygwarts.magical.philosophers_stone	import Transmutable
from pygwarts.irma.access.bookmarks			import VolumeBookmark
from pygwarts.irma.access.handlers			import AccessHandler
from pygwarts.irma.access.volume			import LibraryVolume








class AccessInducer(Transmutable):

	"""
		irma.access core super class for inducing results of access to library. In other words, this object
		is to process all data, gathered during access to library, and produce some report as a string.
		Must be declared under corresponding AccessHandler object, which data will be Induced. In turn
		the AccessHandler object above must be declared under VolumeBookmark object, which will be the
		end point for both, so all data will be anyway stored right there. Basically AccessInducer object
		must do the following upon being called:
			1. access either AccessHandler or LibraryVolume object it assigned to;
			2. process required data the way it supposed to;
			3. induce the result as a string.
		Any inducer must implement it's __call__ which will accept the only argument LibraryVolume object
		as a source file representation, will do some desired processing on the data fetched and will
		return exactly string or None. It is the originally designed idea to return only string, cause
		the default processing of inducers in VolumeBookmark.view method relies on concatenation of
		inducers strings.
	"""

	__call__ :Callable[[LibraryVolume],str | None]

	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)


		if	isinstance(handler := getattr(self, "_UPPER_LAYER", None), AccessHandler):
			if	isinstance(booklink := getattr(handler, "_UPPER_LAYER", None), VolumeBookmark):


				booklink(self, "inducer")
				self.loggy.debug(f"Assigned to {booklink} bookmark as {handler} inducer")


			else:	self.loggy.info(f"AccessInducer {self} not assigned to any bookmark")
		else:		self.loggy.info(f"AccessInducer {self} not registered to any handler")




	def get_handler_counter(self) -> int | None :

		"""
			Helper method that accesses the AccessHandler object it is assigned to, so it must be it's
			"_UPPER_LAYER", and returns it's "access_handler_counter" attribute, which must be integer
			that represents the counter for AccessHandler invocations. Returns None if either current
			AccessInducer object assigned not to AccessHandler, or if AccessHandler has no or not integer
			"access_handler_counter".
		"""

		if	isinstance(handler := getattr(self, "_UPPER_LAYER", None), AccessHandler):
			if	isinstance(handler_counter := getattr(handler, "access_handler_counter", None), int):


				self.loggy.debug(f"Found {handler} access handler counter {handler_counter}")
				return handler_counter
		else:	self.loggy.debug(f"Not registered to any handler")




	def get_register_counter(self, volume :LibraryVolume) -> int | None :

		"""
			Helper method that accesses provided "volume" argument LibraryVolume object with AccessHandler
			object it is assigned to, so it must be it's "_UPPER_LAYER", and returns corresponding mapped
			"counter" value, which must be integer that represents the counter of AccessHandler invocations
			for exactly "volume" object. Returns None if either current AccessInducer object assigned not
			to AccessHandler or "volume" is not LibraryVolume object, or if AccessHandler has no or not
			integer "counter" mapping in "volume".
		"""

		if	isinstance(handler := getattr(self, "_UPPER_LAYER", None), AccessHandler):
			if	isinstance(volume, LibraryVolume) and isinstance(vollink := volume[handler], dict):
				if	isinstance(register_counter := vollink.get("counter"), int):


					self.loggy.debug(f"Found {handler} register counter {register_counter}")
					return register_counter
			else:	self.loggy.debug(f"Handler {handler} not registered to volume {volume}")
		else:		self.loggy.debug(f"Not registered to any handler")




	def get_register_recap(self, volume :LibraryVolume) -> Any | None :

		"""
			Helper method that accesses provided "volume" argument LibraryVolume object with AccessHandler
			object it is assigned to, so it must be it's "_UPPER_LAYER", and returns corresponding mapped
			"recap" value, which might be any type and represents the object AccessHandler maintains for
			"volume". Returns None if either current AccessInducer object assigned not to AccessHandler
			or "volume" is not LibraryVolume object, or if AccessHandler has no or None "recap" mapping
			in "volume".
		"""

		if	isinstance(handler := getattr(self, "_UPPER_LAYER", None), AccessHandler):
			if	isinstance(volume, LibraryVolume) and isinstance(vollink := volume[handler], dict):
				if	(handler_recap := vollink.get("recap")) is not None:


					self.loggy.debug(f"Found {handler} recap of {type(handler_recap)}")
					return handler_recap
			else:	self.loggy.debug(f"Handler {handler} not registered to volume {volume}")
		else:		self.loggy.debug(f"Not registered to any handler")










from os								import path as ospath
from typing							import Optional
from typing							import Callable
from pygwarts.magical.spells		import patronus
from pygwarts.hedwig.mail.letter	import AddressField
from pygwarts.hedwig.mail.letter	import StringField
from pygwarts.hedwig.mail.letter	import TextField








class SenderField(AddressField):
	def __init__(self, *args, **kwargs):

		super().__init__(*args, **kwargs)
		self.make_field("sender")


class RecipientField(AddressField):
	def __init__(self, *args, **kwargs):

		super().__init__(*args, **kwargs)
		self.make_field("recipient")


class ccField(AddressField):
	def __init__(self, *args, **kwargs):

		super().__init__(*args, **kwargs)
		self.make_field("cc")


class bccField(AddressField):
	def __init__(self, *args, **kwargs):

		super().__init__(*args, **kwargs)
		self.make_field("bcc")


class SubjectField(TextField):
	def __init__(self, *args, **kwargs):

		super().__init__(*args, **kwargs)
		self.make_field("subject")


class BodyField(TextField):
	def __init__(self, *args, **kwargs):

		super().__init__(*args, **kwargs)
		self.make_field("body")








class AttachmentField(StringField):

	"""
		hedwig.mail optional letter field class that implements handling of attachment files. It is a
		substantive letter field, which main purpose is to obtain a string "field_value", which normally
		must represent the attachment file path, verify it points to a real file and return it, so the
		letter builder will be able to attach it to the letter. Optional "modifiers" callable might
		accept "field_value" in order to make some processing beforehand and return the final file path.
		If any Exception will be raised when "modifiers" is called, it will be logged and original
		"field_value" will be considered as a final file path, means it will be the subject for
		verification. This is crucial moment for "modifiers", as if provided it will first intercept
		"field_value", so e.g. "field_value" might originally be not a real file path but some string,
		which when fed to a "modifiers" will output the real file path, may be even made by "modifiers".
		So it must be accounted that if such manipulation will fail, the original non-file "field_value"
		will not pass verification and no attachment will be provided. The other important thing about
		"modifiers" is that all files manipulation it does goes in letter construction time, so after if
		the letter builder will eventually fail, the impact to file system made by "modifiers" will stay.
		If final string will not pass the verification, or "field_value" is invalid, None will be returned.
	"""


	field_value	:str
	modifiers	:Optional[Callable[[str],str]]


	def __init__(self, *args, **kwargs):


		super().__init__(*args, **kwargs)
		self.make_field("attachment")


	def __call__(self) -> str | None :


		if	(field := super().__call__()) is not None:
			if	callable(getattr(self, "modifiers", None)):


				# As "modifiers" will be invoked as a member, it assumed current object reference will
				# be implicitly passed as a first argument to "modifiers", along with the "field_value"
				# as a second argument. Due to such particularity, any "modifiers" that are not supposed
				# initially as a member, e.g. lambda functions, must account to, in better way, gather
				# all arguments and process only the last one.
				try:	final_field = self.modifiers(field)
				except	Exception as E:

					self.loggy.error(f"AttachmentField {self} field failed due to {patronus(E)}")
					final_field = field
			else:

				self.loggy.debug("Modifiers not used")
				final_field = field

			if	ospath.isfile(final_field):

				self.loggy.debug(f"Obtained file path \"{final_field}\"")
				return final_field

			self.loggy.info(f"{self} cannot attach non-existent file \"{final_field}\"")










import	os
import	shutil
from	pathlib			import Path
from	pygwarts.tests	import PygwartsTestCase








class HagridTestCase(PygwartsTestCase):

	"""
		Super class for hagrid test cases
	"""

	HAGRID_ROOT			= Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid"

	INIT_SET_BOUGH_1	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_init_dst1")
	INIT_SET_BOUGH_2	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_init_dst2")
	INIT_SET_BOUGH_3	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_init_dst3")
	INIT_SET_SPROUT		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_init_src")
	INIT_HANDLER		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_init.loggy")
	ORDER_SET_BOUGH_1	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_order_dst1")
	ORDER_SET_BOUGH_2	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_order_dst2")
	ORDER_SET_BOUGH_3	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_order_dst3")
	PEEL_HANDLER		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_peels.loggy")
	DRAFT_PEEK_HANDLER	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_dpeeks.loggy")
	BLIND_PEEK_HANDLER	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_bpeeks.loggy")
	SIFT_HANDLER		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_sift.loggy")
	FLOURISH_HANDLER	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_flou.loggy")

	EASY_SET_BOUGH		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy_dst")
	EASY_SET_SPROUT		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy_src")
	EASY_SET_SEEDS		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy.seeds")
	EASY_HANDLER_1		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy1.loggy")
	EASY_HANDLER_2		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy2.loggy")
	EASY_HANDLER_3		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy3.loggy")
	EASY_HANDLER_4		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy4.loggy")
	EASY_HANDLER_5		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy5.loggy")
	EASY_HANDLER_6		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy6.loggy")
	EASY_HANDLER_7		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy7.loggy")
	EASY_HANDLER_8		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy8.loggy")
	EASY_HANDLER_9		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy9.loggy")
	EASY_HANDLER_10		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy10.loggy")
	EASY_HANDLER_11		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy11.loggy")
	EASY_HANDLER_12		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy12.loggy")
	EASY_HANDLER_13		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy13.loggy")
	EASY_HANDLER_15		= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_easy15.loggy")

	MEDIUM_SET_BOUGH_1	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium_dst1")
	MEDIUM_SET_BOUGH_2	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium_dst2")
	MEDIUM_SET_BOUGH_3	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium_dst3")
	MEDIUM_SET_SPROUT_1	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium_src1")
	MEDIUM_SET_SPROUT_2	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium_src2")
	MEDIUM_SET_SPROUT_3	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium_src3")
	MEDIUM_SET_SEEDS	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium.seeds")
	MEDIUM_HANDLER_1	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium1.loggy")
	MEDIUM_HANDLER_2	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium2.loggy")
	MEDIUM_HANDLER_3	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium3.loggy")
	MEDIUM_HANDLER_4	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium4.loggy")
	MEDIUM_HANDLER_5	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium5.loggy")
	MEDIUM_HANDLER_6	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium6.loggy")
	MEDIUM_HANDLER_7	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium7.loggy")
	MEDIUM_HANDLER_8	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium8.loggy")
	MEDIUM_HANDLER_9	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium9.loggy")
	MEDIUM_HANDLER_10	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium10.loggy")
	MEDIUM_HANDLER_11	= str(Path(PygwartsTestCase._PygwartsTestCase__CWD) /"hagrid" /"unit_medium11.loggy")
















class EasySet(HagridTestCase):

	"""
		Super for easy cases, like single sprout and single bough
	"""

	@classmethod
	def setUpClass(cls):

		"""
			Following operation must make such a tree:
			d- EASY_SET_SPROUT
				f- WitchDoctor.txt
				d- pros
			    	f- not so good.txt
			    	f- good.txt
			    	f- good so much.txt
			    	d- redundant_folder_2
			        	f- redundant_file_2
				d- cons
					f- main.txt
					f- not_main.txt
				d- redundant_folder_1
					f- redundant_file_1
		"""

		cls.file1					= os.path.join(cls.EASY_SET_SPROUT, "WitchDoctor.txt")
		cls.pros_folder				= os.path.join(cls.EASY_SET_SPROUT, "pros")
		cls.pros_file1				= os.path.join(cls.pros_folder, "not so good.txt")
		cls.pros_file2				= os.path.join(cls.pros_folder, "good.txt")
		cls.pros_file3				= os.path.join(cls.pros_folder, "good so much.txt")
		cls.cons_folder				= os.path.join(cls.EASY_SET_SPROUT, "cons")
		cls.cons_file1				= os.path.join(cls.cons_folder, "main.txt")
		cls.cons_file2				= os.path.join(cls.cons_folder, "not_main.txt")
		cls.redundant_1_folder		= os.path.join(cls.EASY_SET_SPROUT, "redundant_folder_1")
		cls.redundant_2_folder		= os.path.join(cls.EASY_SET_SPROUT, "pros", "redundant_folder_2")
		cls.redundant_1				= os.path.join(cls.redundant_1_folder, "redundant_file_1")
		cls.redundant_2				= os.path.join(cls.redundant_2_folder, "redundant_file_2")

		cls.dst_file1				= os.path.join(cls.EASY_SET_BOUGH, "WitchDoctor.txt")
		cls.dst_pros_folder			= os.path.join(cls.EASY_SET_BOUGH, "pros")
		cls.dst_pros_file1			= os.path.join(cls.dst_pros_folder, "not so good.txt")
		cls.dst_pros_file2			= os.path.join(cls.dst_pros_folder, "good.txt")
		cls.dst_pros_file3			= os.path.join(cls.dst_pros_folder, "good so much.txt")
		cls.dst_cons_folder			= os.path.join(cls.EASY_SET_BOUGH, "cons")
		cls.dst_cons_file1			= os.path.join(cls.dst_cons_folder, "main.txt")
		cls.dst_cons_file2			= os.path.join(cls.dst_cons_folder, "not_main.txt")
		cls.dst_redundant_1_folder	= os.path.join(cls.EASY_SET_BOUGH, "redundant_folder_1")
		cls.dst_redundant_2_folder	= os.path.join(cls.EASY_SET_BOUGH, "pros", "redundant_folder_2")
		cls.dst_redundant_1			= os.path.join(cls.dst_redundant_1_folder, "redundant_file_1")
		cls.dst_redundant_2			= os.path.join(cls.dst_redundant_2_folder, "redundant_file_2")

		cls.clean(cls)
		cls.fmake(cls, cls.file1, "OOH EEH OOH AH AH TING TANG WALA WALA BING BANG")
		cls.fmake(cls, cls.pros_file1, "may be ain't best way")
		cls.fmake(cls, cls.pros_file2, "probably the best way")
		cls.fmake(cls, cls.pros_file3, "definitely the best way")

		if	not os.path.isdir(cls.EASY_SET_BOUGH):	os.makedirs(cls.EASY_SET_BOUGH)
		if	not os.path.isfile(cls.cons_file1) :	cls.fmake(cls, cls.cons_file1, "might cause a headache")
		if	not os.path.isfile(cls.cons_file2) :	cls.fmake(cls, cls.cons_file2, "annihilation")
		if	not os.path.isfile(cls.redundant_1):	cls.fmake(cls, cls.redundant_1, "no use 1")
		if	not os.path.isfile(cls.redundant_2):	cls.fmake(cls, cls.redundant_2, "no use 2")

	def clean(self):

		if	os.path.isdir(self.EASY_SET_SPROUT):			shutil.rmtree(self.EASY_SET_SPROUT)
		if	os.path.isdir(self.EASY_SET_BOUGH):				shutil.rmtree(self.EASY_SET_BOUGH)
		if	os.path.isfile(self.EASY_SET_SEEDS):			os.remove(self.EASY_SET_SEEDS)
		if	os.path.isfile(self.EASY_SET_SEEDS + ".db"):	os.remove(self.EASY_SET_SEEDS + ".db")
		if	os.path.isfile(self.EASY_SET_SEEDS + ".dat"):	os.remove(self.EASY_SET_SEEDS + ".dat")
		if	os.path.isfile(self.EASY_SET_SEEDS + ".bak"):	os.remove(self.EASY_SET_SEEDS + ".bak")
		if	os.path.isfile(self.EASY_SET_SEEDS + ".dir"):	os.remove(self.EASY_SET_SEEDS + ".dir")
















class MediumSet(HagridTestCase):

	"""
		Big tests for multi sprout/bough
	"""

	@classmethod
	def setUpClass(cls):

		"""
			d- MEDIUM_SET_SPROUT_1
				d- men due
					f- rock
					f- paper
					f- scissors
					d- girls cries
						f- sad movies
						f- dumbasses (literaly)
						f- onion					- this one to be involved
				d- commons							- collection of empty folders to test Germination
					d- good
					d- not good
					d- bad
					d- not bad
					d- not not
					d- bad bad
					d- bad good
					d- good bad
			d- MEDIUM_SET_SPROUT_2
				f- tasks schedule.txt
				f- ex.plan graph.png
				d- chores
					f- hard-cleaning.task
					f- hard-washing.task
					f- medium-cooking.task
					f- medium-homework.task
					f- medium-physicals.task
					f- easy-procrastinating.task
					f- easy-reflections.task
					f- easy-philosophizing.task
			d- MEDIUM_SET_SPROUT_3
				d- kitchen
					d- fridge
						f- milk
						f- cheese
						f- meat
					d- oven
						f- chicken
						f- pie
					d- table
						d- bread container
							f- bread
							f- crumb1
							f- crumb2
							f- crumb420				- this two crumbs are looking too cool,
							f- crumb69				- so mb sift them out?
			d- MEDIUM_SET_BOUGH_1
				f- flower.weed
			d- MEDIUM_SET_BOUGH_2
				d- weeds
			d- MEDIUM_SET_BOUGH_3
				f- almost.weed						- this one must be excluded from trimming
				d- all_weeds
				d- tools							- this one must be excluded from trimming
		"""

		################################### First sprout ###################################
		cls.d_mss1_md						= os.path.join(cls.MEDIUM_SET_SPROUT_1, "men due")
		cls.f_mss1_md_rock					= os.path.join(cls.d_mss1_md, "rock")
		cls.f_mss1_md_paper					= os.path.join(cls.d_mss1_md, "paper")
		cls.f_mss1_md_scissors				= os.path.join(cls.d_mss1_md, "scissors")
		cls.d_mss1_md_gc					= os.path.join(cls.d_mss1_md, "girls cries")
		cls.f_mss1_md_gc_sadmovies			= os.path.join(cls.d_mss1_md_gc, "sad movies")
		cls.f_mss1_md_gc_dumbasses			= os.path.join(cls.d_mss1_md_gc, "dumbasses (literaly)")
		cls.f_mss1_md_gc_onion				= os.path.join(cls.d_mss1_md_gc, "onion")
		cls.d_mss1_c						= os.path.join(cls.MEDIUM_SET_SPROUT_1, "commons")
		cls.d_mss1_c_good					= os.path.join(cls.d_mss1_c, "good")
		cls.d_mss1_c_notgood				= os.path.join(cls.d_mss1_c, "not good")
		cls.d_mss1_c_bad					= os.path.join(cls.d_mss1_c, "bad")
		cls.d_mss1_c_notbad					= os.path.join(cls.d_mss1_c, "not bad")
		cls.d_mss1_c_notnot					= os.path.join(cls.d_mss1_c, "not not")
		cls.d_mss1_c_badbad					= os.path.join(cls.d_mss1_c, "bad bad")
		cls.d_mss1_c_badgood				= os.path.join(cls.d_mss1_c, "bad good")
		cls.d_mss1_c_goodbad				= os.path.join(cls.d_mss1_c, "good bad")




		################################### Second sprout ###################################
		cls.f_mss2_tasksschedule			= os.path.join(cls.MEDIUM_SET_SPROUT_2, "tasks schedule.txt")
		cls.f_mss2_explangraph				= os.path.join(cls.MEDIUM_SET_SPROUT_2, "ex.plan graph.png")
		cls.d_mss2_c						= os.path.join(cls.MEDIUM_SET_SPROUT_2, "chores")
		cls.f_mss2_c_hardcleaning			= os.path.join(cls.d_mss2_c, "hard-cleaning.task")
		cls.f_mss2_c_hardwashing			= os.path.join(cls.d_mss2_c, "hard-washing.task")
		cls.f_mss2_c_mediumcooking			= os.path.join(cls.d_mss2_c, "medium-cooking.task")
		cls.f_mss2_c_mediumhomework			= os.path.join(cls.d_mss2_c, "medium-homework.task")
		cls.f_mss2_c_mediumphysicals		= os.path.join(cls.d_mss2_c, "medium-physicals.task")
		cls.f_mss2_c_easyprocrastinating	= os.path.join(cls.d_mss2_c, "easy-procrastinating.task")
		cls.f_mss2_c_easyreflections		= os.path.join(cls.d_mss2_c, "easy-reflections.task")
		cls.f_mss2_c_easyphilosophizing		= os.path.join(cls.d_mss2_c, "easy-philosophizing.task")




		################################### Third sprout ###################################
		cls.d_mss3_k						= os.path.join(cls.MEDIUM_SET_SPROUT_3, "kitchen")
		cls.d_mss3_k_f						= os.path.join(cls.d_mss3_k, "fridge")
		cls.f_mss3_k_f_milk					= os.path.join(cls.d_mss3_k_f, "milk")
		cls.f_mss3_k_f_cheese				= os.path.join(cls.d_mss3_k_f, "cheese")
		cls.f_mss3_k_f_meat					= os.path.join(cls.d_mss3_k_f, "meat")
		cls.d_mss3_k_o						= os.path.join(cls.d_mss3_k, "oven")
		cls.f_mss3_k_o_chicken				= os.path.join(cls.d_mss3_k_o, "chicken")
		cls.f_mss3_k_o_pie					= os.path.join(cls.d_mss3_k_o, "pie")
		cls.d_mss3_k_t						= os.path.join(cls.d_mss3_k, "table")
		cls.d_mss3_k_t_bc					= os.path.join(cls.d_mss3_k_t, "bread container")
		cls.f_mss3_k_t_bc_bread				= os.path.join(cls.d_mss3_k_t_bc, "bread")
		cls.f_mss3_k_t_bc_crumb1			= os.path.join(cls.d_mss3_k_t_bc, "crumb1")
		cls.f_mss3_k_t_bc_crumb2			= os.path.join(cls.d_mss3_k_t_bc, "crumb2")
		cls.f_mss3_k_t_bc_crumb420			= os.path.join(cls.d_mss3_k_t_bc, "crumb420")
		cls.f_mss3_k_t_bc_crumb69			= os.path.join(cls.d_mss3_k_t_bc, "crumb69")




		################################### Boughs start ###################################
		cls.f_msb1_flower					= os.path.join(cls.MEDIUM_SET_BOUGH_1, "flower.weed")
		cls.d_msb2_w						= os.path.join(cls.MEDIUM_SET_BOUGH_2, "weeds")
		cls.f_msb3_almost					= os.path.join(cls.MEDIUM_SET_BOUGH_3, "almost.weed")
		cls.d_msb3_a						= os.path.join(cls.MEDIUM_SET_BOUGH_3, "all_weeds")
		cls.d_msb3_t						= os.path.join(cls.MEDIUM_SET_BOUGH_3, "tools")




		################################### First sprout first bough ###################################
		cls.tg1_d_mss1_md					= os.path.join(cls.MEDIUM_SET_BOUGH_1, "men due")
		cls.tg1_f_mss1_md_rock				= os.path.join(cls.tg1_d_mss1_md, "rock")
		cls.tg1_f_mss1_md_paper				= os.path.join(cls.tg1_d_mss1_md, "paper")
		cls.tg1_f_mss1_md_scissors			= os.path.join(cls.tg1_d_mss1_md, "scissors")
		cls.tg1_d_mss1_md_gc				= os.path.join(cls.tg1_d_mss1_md, "girls cries")
		cls.tg1_f_mss1_md_gc_sadmovies		= os.path.join(cls.tg1_d_mss1_md_gc, "sad movies")
		cls.tg1_f_mss1_md_gc_dumbasses		= os.path.join(cls.tg1_d_mss1_md_gc, "dumbasses (literaly)")
		cls.tg1_f_mss1_md_gc_onion			= os.path.join(cls.tg1_d_mss1_md_gc, "onion")
		cls.tg1_d_mss1_c					= os.path.join(cls.MEDIUM_SET_BOUGH_1, "commons")
		cls.tg1_d_mss1_c_good				= os.path.join(cls.tg1_d_mss1_c, "good")
		cls.tg1_d_mss1_c_notgood			= os.path.join(cls.tg1_d_mss1_c, "not good")
		cls.tg1_d_mss1_c_bad				= os.path.join(cls.tg1_d_mss1_c, "bad")
		cls.tg1_d_mss1_c_notbad				= os.path.join(cls.tg1_d_mss1_c, "not bad")
		cls.tg1_d_mss1_c_notnot				= os.path.join(cls.tg1_d_mss1_c, "not not")
		cls.tg1_d_mss1_c_badbad				= os.path.join(cls.tg1_d_mss1_c, "bad bad")
		cls.tg1_d_mss1_c_badgood			= os.path.join(cls.tg1_d_mss1_c, "bad good")
		cls.tg1_d_mss1_c_goodbad			= os.path.join(cls.tg1_d_mss1_c, "good bad")




		################################### Second sprout first bough ###################################
		cls.tg1_f_mss2_tasksschedule		= os.path.join(cls.MEDIUM_SET_BOUGH_1, "tasks schedule.txt")
		cls.tg1_f_mss2_explangraph			= os.path.join(cls.MEDIUM_SET_BOUGH_1, "ex.plan graph.png")
		cls.tg1_d_mss2_c					= os.path.join(cls.MEDIUM_SET_BOUGH_1, "chores")
		cls.tg1_f_mss2_c_hardcleaning		= os.path.join(cls.tg1_d_mss2_c, "hard-cleaning.task")
		cls.tg1_f_mss2_c_hardwashing		= os.path.join(cls.tg1_d_mss2_c, "hard-washing.task")
		cls.tg1_f_mss2_c_mediumcooking		= os.path.join(cls.tg1_d_mss2_c, "medium-cooking.task")
		cls.tg1_f_mss2_c_mediumhomework		= os.path.join(cls.tg1_d_mss2_c, "medium-homework.task")
		cls.tg1_f_mss2_c_mediumphysicals	= os.path.join(cls.tg1_d_mss2_c, "medium-physicals.task")
		cls.tg1_f_mss2_c_easyprocrastinating= os.path.join(cls.tg1_d_mss2_c, "easy-procrastinating.task")
		cls.tg1_f_mss2_c_easyreflections	= os.path.join(cls.tg1_d_mss2_c, "easy-reflections.task")
		cls.tg1_f_mss2_c_easyphilosophizing	= os.path.join(cls.tg1_d_mss2_c, "easy-philosophizing.task")




		################################### Third sprout first bough ###################################
		cls.tg1_d_mss3_k					= os.path.join(cls.MEDIUM_SET_BOUGH_1, "kitchen")
		cls.tg1_d_mss3_k_f					= os.path.join(cls.tg1_d_mss3_k, "fridge")
		cls.tg1_f_mss3_k_f_milk				= os.path.join(cls.tg1_d_mss3_k_f, "milk")
		cls.tg1_f_mss3_k_f_cheese			= os.path.join(cls.tg1_d_mss3_k_f, "cheese")
		cls.tg1_f_mss3_k_f_meat				= os.path.join(cls.tg1_d_mss3_k_f, "meat")
		cls.tg1_d_mss3_k_o					= os.path.join(cls.tg1_d_mss3_k, "oven")
		cls.tg1_f_mss3_k_o_chicken			= os.path.join(cls.tg1_d_mss3_k_o, "chicken")
		cls.tg1_f_mss3_k_o_pie				= os.path.join(cls.tg1_d_mss3_k_o, "pie")
		cls.tg1_d_mss3_k_t					= os.path.join(cls.tg1_d_mss3_k, "table")
		cls.tg1_d_mss3_k_t_bc				= os.path.join(cls.tg1_d_mss3_k_t, "bread container")
		cls.tg1_f_mss3_k_t_bc_bread			= os.path.join(cls.tg1_d_mss3_k_t_bc, "bread")
		cls.tg1_f_mss3_k_t_bc_crumb1		= os.path.join(cls.tg1_d_mss3_k_t_bc, "crumb1")
		cls.tg1_f_mss3_k_t_bc_crumb2		= os.path.join(cls.tg1_d_mss3_k_t_bc, "crumb2")
		cls.tg1_f_mss3_k_t_bc_crumb420		= os.path.join(cls.tg1_d_mss3_k_t_bc, "crumb420")
		cls.tg1_f_mss3_k_t_bc_crumb69		= os.path.join(cls.tg1_d_mss3_k_t_bc, "crumb69")




		################################### First sprout second bough ###################################
		cls.tg2_d_mss1_md					= os.path.join(cls.MEDIUM_SET_BOUGH_2, "men due")
		cls.tg2_f_mss1_md_rock				= os.path.join(cls.tg2_d_mss1_md, "rock")
		cls.tg2_f_mss1_md_paper				= os.path.join(cls.tg2_d_mss1_md, "paper")
		cls.tg2_f_mss1_md_scissors			= os.path.join(cls.tg2_d_mss1_md, "scissors")
		cls.tg2_d_mss1_md_gc				= os.path.join(cls.tg2_d_mss1_md, "girls cries")
		cls.tg2_f_mss1_md_gc_sadmovies		= os.path.join(cls.tg2_d_mss1_md_gc, "sad movies")
		cls.tg2_f_mss1_md_gc_dumbasses		= os.path.join(cls.tg2_d_mss1_md_gc, "dumbasses (literaly)")
		cls.tg2_f_mss1_md_gc_onion			= os.path.join(cls.tg2_d_mss1_md_gc, "onion")
		cls.tg2_d_mss1_c					= os.path.join(cls.MEDIUM_SET_BOUGH_2, "commons")
		cls.tg2_d_mss1_c_good				= os.path.join(cls.tg2_d_mss1_c, "good")
		cls.tg2_d_mss1_c_notgood			= os.path.join(cls.tg2_d_mss1_c, "not good")
		cls.tg2_d_mss1_c_bad				= os.path.join(cls.tg2_d_mss1_c, "bad")
		cls.tg2_d_mss1_c_notbad				= os.path.join(cls.tg2_d_mss1_c, "not bad")
		cls.tg2_d_mss1_c_notnot				= os.path.join(cls.tg2_d_mss1_c, "not not")
		cls.tg2_d_mss1_c_badbad				= os.path.join(cls.tg2_d_mss1_c, "bad bad")
		cls.tg2_d_mss1_c_badgood			= os.path.join(cls.tg2_d_mss1_c, "bad good")
		cls.tg2_d_mss1_c_goodbad			= os.path.join(cls.tg2_d_mss1_c, "good bad")




		################################### Second sprout second bough ###################################
		cls.tg2_f_mss2_tasksschedule		= os.path.join(cls.MEDIUM_SET_BOUGH_2, "tasks schedule.txt")
		cls.tg2_f_mss2_explangraph			= os.path.join(cls.MEDIUM_SET_BOUGH_2, "ex.plan graph.png")
		cls.tg2_d_mss2_c					= os.path.join(cls.MEDIUM_SET_BOUGH_2, "chores")
		cls.tg2_f_mss2_c_hardcleaning		= os.path.join(cls.tg2_d_mss2_c, "hard-cleaning.task")
		cls.tg2_f_mss2_c_hardwashing		= os.path.join(cls.tg2_d_mss2_c, "hard-washing.task")
		cls.tg2_f_mss2_c_mediumcooking		= os.path.join(cls.tg2_d_mss2_c, "medium-cooking.task")
		cls.tg2_f_mss2_c_mediumhomework		= os.path.join(cls.tg2_d_mss2_c, "medium-homework.task")
		cls.tg2_f_mss2_c_mediumphysicals	= os.path.join(cls.tg2_d_mss2_c, "medium-physicals.task")
		cls.tg2_f_mss2_c_easyprocrastinating= os.path.join(cls.tg2_d_mss2_c, "easy-procrastinating.task")
		cls.tg2_f_mss2_c_easyreflections	= os.path.join(cls.tg2_d_mss2_c, "easy-reflections.task")
		cls.tg2_f_mss2_c_easyphilosophizing	= os.path.join(cls.tg2_d_mss2_c, "easy-philosophizing.task")




		################################### Third sprout second bough ###################################
		cls.tg2_d_mss3_k					= os.path.join(cls.MEDIUM_SET_BOUGH_2, "kitchen")
		cls.tg2_d_mss3_k_f					= os.path.join(cls.tg2_d_mss3_k, "fridge")
		cls.tg2_f_mss3_k_f_milk				= os.path.join(cls.tg2_d_mss3_k_f, "milk")
		cls.tg2_f_mss3_k_f_cheese			= os.path.join(cls.tg2_d_mss3_k_f, "cheese")
		cls.tg2_f_mss3_k_f_meat				= os.path.join(cls.tg2_d_mss3_k_f, "meat")
		cls.tg2_d_mss3_k_o					= os.path.join(cls.tg2_d_mss3_k, "oven")
		cls.tg2_f_mss3_k_o_chicken			= os.path.join(cls.tg2_d_mss3_k_o, "chicken")
		cls.tg2_f_mss3_k_o_pie				= os.path.join(cls.tg2_d_mss3_k_o, "pie")
		cls.tg2_d_mss3_k_t					= os.path.join(cls.tg2_d_mss3_k, "table")
		cls.tg2_d_mss3_k_t_bc				= os.path.join(cls.tg2_d_mss3_k_t, "bread container")
		cls.tg2_f_mss3_k_t_bc_bread			= os.path.join(cls.tg2_d_mss3_k_t_bc, "bread")
		cls.tg2_f_mss3_k_t_bc_crumb1		= os.path.join(cls.tg2_d_mss3_k_t_bc, "crumb1")
		cls.tg2_f_mss3_k_t_bc_crumb2		= os.path.join(cls.tg2_d_mss3_k_t_bc, "crumb2")
		cls.tg2_f_mss3_k_t_bc_crumb420		= os.path.join(cls.tg2_d_mss3_k_t_bc, "crumb420")
		cls.tg2_f_mss3_k_t_bc_crumb69		= os.path.join(cls.tg2_d_mss3_k_t_bc, "crumb69")




		################################### First sprout third bough ###################################
		cls.tg3_d_mss1_md					= os.path.join(cls.MEDIUM_SET_BOUGH_3, "men due")
		cls.tg3_f_mss1_md_rock				= os.path.join(cls.tg3_d_mss1_md, "rock")
		cls.tg3_f_mss1_md_paper				= os.path.join(cls.tg3_d_mss1_md, "paper")
		cls.tg3_f_mss1_md_scissors			= os.path.join(cls.tg3_d_mss1_md, "scissors")
		cls.tg3_d_mss1_md_gc				= os.path.join(cls.tg3_d_mss1_md, "girls cries")
		cls.tg3_f_mss1_md_gc_sadmovies		= os.path.join(cls.tg3_d_mss1_md_gc, "sad movies")
		cls.tg3_f_mss1_md_gc_dumbasses		= os.path.join(cls.tg3_d_mss1_md_gc, "dumbasses (literaly)")
		cls.tg3_f_mss1_md_gc_onion			= os.path.join(cls.tg3_d_mss1_md_gc, "onion")
		cls.tg3_d_mss1_c					= os.path.join(cls.MEDIUM_SET_BOUGH_3, "commons")
		cls.tg3_d_mss1_c_good				= os.path.join(cls.tg3_d_mss1_c, "good")
		cls.tg3_d_mss1_c_notgood			= os.path.join(cls.tg3_d_mss1_c, "not good")
		cls.tg3_d_mss1_c_bad				= os.path.join(cls.tg3_d_mss1_c, "bad")
		cls.tg3_d_mss1_c_notbad				= os.path.join(cls.tg3_d_mss1_c, "not bad")
		cls.tg3_d_mss1_c_notnot				= os.path.join(cls.tg3_d_mss1_c, "not not")
		cls.tg3_d_mss1_c_badbad				= os.path.join(cls.tg3_d_mss1_c, "bad bad")
		cls.tg3_d_mss1_c_badgood			= os.path.join(cls.tg3_d_mss1_c, "bad good")
		cls.tg3_d_mss1_c_goodbad			= os.path.join(cls.tg3_d_mss1_c, "good bad")




		################################### Sceond sprout third bough ###################################
		cls.tg3_f_mss2_tasksschedule		= os.path.join(cls.MEDIUM_SET_BOUGH_3, "tasks schedule.txt")
		cls.tg3_f_mss2_explangraph			= os.path.join(cls.MEDIUM_SET_BOUGH_3, "ex.plan graph.png")
		cls.tg3_d_mss2_c					= os.path.join(cls.MEDIUM_SET_BOUGH_3, "chores")
		cls.tg3_f_mss2_c_hardcleaning		= os.path.join(cls.tg3_d_mss2_c, "hard-cleaning.task")
		cls.tg3_f_mss2_c_hardwashing		= os.path.join(cls.tg3_d_mss2_c, "hard-washing.task")
		cls.tg3_f_mss2_c_mediumcooking		= os.path.join(cls.tg3_d_mss2_c, "medium-cooking.task")
		cls.tg3_f_mss2_c_mediumhomework		= os.path.join(cls.tg3_d_mss2_c, "medium-homework.task")
		cls.tg3_f_mss2_c_mediumphysicals	= os.path.join(cls.tg3_d_mss2_c, "medium-physicals.task")
		cls.tg3_f_mss2_c_easyprocrastinating= os.path.join(cls.tg3_d_mss2_c, "easy-procrastinating.task")
		cls.tg3_f_mss2_c_easyreflections	= os.path.join(cls.tg3_d_mss2_c, "easy-reflections.task")
		cls.tg3_f_mss2_c_easyphilosophizing	= os.path.join(cls.tg3_d_mss2_c, "easy-philosophizing.task")




		################################### Third sprout third bough ###################################
		cls.tg3_d_mss3_k					= os.path.join(cls.MEDIUM_SET_BOUGH_3, "kitchen")
		cls.tg3_d_mss3_k_f					= os.path.join(cls.tg3_d_mss3_k, "fridge")
		cls.tg3_f_mss3_k_f_milk				= os.path.join(cls.tg3_d_mss3_k_f, "milk")
		cls.tg3_f_mss3_k_f_cheese			= os.path.join(cls.tg3_d_mss3_k_f, "cheese")
		cls.tg3_f_mss3_k_f_meat				= os.path.join(cls.tg3_d_mss3_k_f, "meat")
		cls.tg3_d_mss3_k_o					= os.path.join(cls.tg3_d_mss3_k, "oven")
		cls.tg3_f_mss3_k_o_chicken			= os.path.join(cls.tg3_d_mss3_k_o, "chicken")
		cls.tg3_f_mss3_k_o_pie				= os.path.join(cls.tg3_d_mss3_k_o, "pie")
		cls.tg3_d_mss3_k_t					= os.path.join(cls.tg3_d_mss3_k, "table")
		cls.tg3_d_mss3_k_t_bc				= os.path.join(cls.tg3_d_mss3_k_t, "bread container")
		cls.tg3_f_mss3_k_t_bc_bread			= os.path.join(cls.tg3_d_mss3_k_t_bc, "bread")
		cls.tg3_f_mss3_k_t_bc_crumb1		= os.path.join(cls.tg3_d_mss3_k_t_bc, "crumb1")
		cls.tg3_f_mss3_k_t_bc_crumb2		= os.path.join(cls.tg3_d_mss3_k_t_bc, "crumb2")
		cls.tg3_f_mss3_k_t_bc_crumb420		= os.path.join(cls.tg3_d_mss3_k_t_bc, "crumb420")
		cls.tg3_f_mss3_k_t_bc_crumb69		= os.path.join(cls.tg3_d_mss3_k_t_bc, "crumb69")




		cls.clean(cls)
		################################### First sprout files ###################################
		cls.fmake(cls, cls.f_mss1_md_rock, "it's about time it's about power")
		cls.fmake(cls, cls.f_mss1_md_paper, "if cut it's like the face insight is right beneath my skin")
		cls.fmake(cls, cls.f_mss1_md_scissors, "only in HD or better")
		cls.fmake(cls, cls.f_mss1_md_gc_sadmovies, "green mile, knockin' on heaven's door")
		cls.fmake(cls, cls.f_mss1_md_gc_dumbasses, "HA-HA, CLASSIC")
		cls.fmake(cls, cls.f_mss1_md_gc_onion, "makes even devil cry")
		os.makedirs(cls.d_mss1_c_good)
		os.makedirs(cls.d_mss1_c_notgood)
		os.makedirs(cls.d_mss1_c_bad)
		os.makedirs(cls.d_mss1_c_notbad)
		os.makedirs(cls.d_mss1_c_notnot)
		os.makedirs(cls.d_mss1_c_badbad)
		os.makedirs(cls.d_mss1_c_badgood)
		os.makedirs(cls.d_mss1_c_goodbad)




		################################### Second sprout files ###################################
		cls.fmake(cls, cls.f_mss2_tasksschedule, "1. get up\n2. get sad")
		cls.fmake(cls, cls.f_mss2_explangraph, "FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF")
		cls.fmake(cls, cls.f_mss2_c_hardcleaning, "especially vacuum")
		cls.fmake(cls, cls.f_mss2_c_hardwashing, "dishes so dishes")
		cls.fmake(cls, cls.f_mss2_c_mediumcooking, "who do you think i am, a chemist?")
		cls.fmake(cls, cls.f_mss2_c_mediumhomework, "my son homework, ofcourse")
		cls.fmake(cls, cls.f_mss2_c_mediumphysicals, "what the flip is that?")
		cls.fmake(cls, cls.f_mss2_c_easyprocrastinating, "the easiest thing ever")
		cls.fmake(cls, cls.f_mss2_c_easyreflections, "litlle harder but still easy")
		cls.fmake(cls, cls.f_mss2_c_easyphilosophizing, "not easy at all, but why not")




		################################### Third sprout files ###################################
		cls.fmake(cls, cls.f_mss3_k_f_milk, "from cow")
		cls.fmake(cls, cls.f_mss3_k_f_cheese, "from... cow")
		cls.fmake(cls, cls.f_mss3_k_f_meat, "from...")
		cls.fmake(cls, cls.f_mss3_k_o_chicken, "cooked in ~60 minutes")
		cls.fmake(cls, cls.f_mss3_k_o_pie, "already baked")
		cls.fmake(cls, cls.f_mss3_k_t_bc_bread, "always crumbles")
		cls.fmake(cls, cls.f_mss3_k_t_bc_crumb1, "i barely believe it is just the first crumb")
		cls.fmake(cls, cls.f_mss3_k_t_bc_crumb2, "i don't believe it is really the second crumb")
		cls.fmake(cls, cls.f_mss3_k_t_bc_crumb420, "this crumb get really high")
		cls.fmake(cls, cls.f_mss3_k_t_bc_crumb69, "this crumb get really nice")




		################################### First bough ###################################
		cls.fmake(cls, cls.f_msb1_flower, "this flower was beutifull, but now effloresced")
		################################### Second bough ###################################
		os.makedirs(cls.d_msb2_w, exist_ok=True)
		################################### Third bough ###################################
		cls.fmake(cls, cls.f_msb3_almost, "as it's not weed yet, it should be left growing")
		os.makedirs(cls.d_msb3_a, exist_ok=True)
		os.makedirs(cls.d_msb3_t, exist_ok=True)





	def clean(self):

		if	os.path.isdir(self.MEDIUM_SET_SPROUT_1):			shutil.rmtree(self.MEDIUM_SET_SPROUT_1)
		if	os.path.isdir(self.MEDIUM_SET_SPROUT_2):			shutil.rmtree(self.MEDIUM_SET_SPROUT_2)
		if	os.path.isdir(self.MEDIUM_SET_SPROUT_3):			shutil.rmtree(self.MEDIUM_SET_SPROUT_3)
		if	os.path.isdir(self.MEDIUM_SET_BOUGH_1):				shutil.rmtree(self.MEDIUM_SET_BOUGH_1)
		if	os.path.isdir(self.MEDIUM_SET_BOUGH_2):				shutil.rmtree(self.MEDIUM_SET_BOUGH_2)
		if	os.path.isdir(self.MEDIUM_SET_BOUGH_3):				shutil.rmtree(self.MEDIUM_SET_BOUGH_3)
		if	os.path.isfile(self.MEDIUM_SET_SEEDS):				os.remove(self.MEDIUM_SET_SEEDS)
		if	os.path.isfile(self.MEDIUM_SET_SEEDS + ".db"):		os.remove(self.MEDIUM_SET_SEEDS + ".db")
		if	os.path.isfile(self.MEDIUM_SET_SEEDS + ".dat"):		os.remove(self.MEDIUM_SET_SEEDS + ".dat")
		if	os.path.isfile(self.MEDIUM_SET_SEEDS + ".bak"):		os.remove(self.MEDIUM_SET_SEEDS + ".bak")
		if	os.path.isfile(self.MEDIUM_SET_SEEDS + ".dir"):		os.remove(self.MEDIUM_SET_SEEDS + ".dir")







